import {Injectable} from '@angular/core'
import {interval} from 'rxjs'

@Injectable({
  providedIn: 'root'
})
export class AlertService {
  private alertQueue: Alert[] = []
  private activeHandler: AlertHandlerComponent

  constructor() {
    // interval(500).subscribe({next: i => this.pushError('Test error ' + i)})
  }

  pushSuccess(message: String) {
    this.enqueue(AlertType.Success, message)
  }

  pushWarning(message: String) {
    this.enqueue(AlertType.Warning, message)
  }

  pushError(message: String) {
    this.enqueue(AlertType.Error, message)
  }

  set activeAlertHandlerComponent(handler: AlertHandlerComponent) {
    this.activeHandler = handler
  }

  private enqueue(type: AlertType, message: String) {
    const alert = {type, message}
    if (this.activeHandler && !this.activeHandler.isBufferFull) {
      if (this.alertQueue.length == 0) {
        this.activeHandler.pushAlert(alert)
      } else {
        this.activeHandler.pushAlert(this.dequeue())
        this.alertQueue.unshift(alert)
      }
    } else {
      this.alertQueue.unshift(alert)
    }
  }

  private dequeue(): Alert {
    return this.alertQueue.pop()
  }
}

/**
 * An alert handler component is a component that will show the alerts pushed by the alert system to the user.
 */
export abstract class AlertHandlerComponent {
  protected static readonly DEFAULT_ALERT_BUFFER_SIZE = 3
  protected static readonly DEFAULT_ALERT_DURATION = 5

  alertBuffer = new Array<{ alert: Alert, time: number }>()
  protected alertBufferSize: number = AlertHandlerComponent.DEFAULT_ALERT_BUFFER_SIZE
  protected alertDuration: number = AlertHandlerComponent.DEFAULT_ALERT_DURATION
  protected alertDurationCounter = 0

  protected constructor(
    protected alertService: AlertService
  ) {
    alertService.activeAlertHandlerComponent = this

    interval(1000).subscribe({
      next: () => {
        this.alertDurationCounter++
        if (this.alertBuffer.length > 1) {
          this.alertBuffer
            .filter(a => a.time + this.alertDuration < this.alertDurationCounter)
            .map(a => this.alertBuffer.indexOf(a))
            .forEach(i => this.alertBuffer.splice(i, 1))
        }
      }
    })
  }

  pushAlert(alert: Alert) {
    this.alertBuffer.unshift({alert: alert, time: this.alertDurationCounter})
  }

  get isBufferFull(): boolean {
    return this.alertBuffer.length >= this.alertBufferSize
  }
}

/**
 * An alert is a message that will be shown to the user.
 *
 * An alert can be a success alert, a warning alert of an error alert.
 */
class Alert {
  constructor(
    public type: AlertType,
    public message: String
  ) {
  }
}

export enum AlertType {
  Success,
  Warning,
  Error
}
